import subprocess
import sys
from typing import List

import psutil

# Xdo is a pain to use on NixOS due to https://github.com/NixOS/nixpkgs/issues/7307
# so let's just use the xdotool binary instead

"""
Focus an application if it's running or start it if it's not.

NO LONGER TRUE: Must have libxdo-dev installed on Ubuntu and it must be recent (3.2021+).
In old versions, `search_windows` won't return anything here...
"""


def focus_window_of_process(pid: int) -> None:
    window_ids = [
        wid
        for wid in _workaround_get_windows_for_pid(pid)
        if _check_window_not_dropdown(wid)
    ]
    if not window_ids:
        raise ValueError("No windows for application!")

    if len(window_ids) > 1:
        print("!!! more than one window for application")

    window_id = window_ids[0]
    _pull_window_here(window_id)


def _workaround_get_windows_for_pid(pid: int) -> List[int]:
    return [
        int(line.strip())
        for line in subprocess.check_output(
            ["xdotool", "search", "--pid", str(pid)]
        ).splitlines()
        if line.strip()
    ]


def _get_current_desktop() -> int:
    return int(subprocess.check_output(["xdotool", "get_desktop"]).strip())


def _pull_window_here(window_id: int) -> None:
    desktop = _get_current_desktop()
    subprocess.check_call(
        [
            "xdotool",
            "set_desktop_for_window",
            str(window_id),
            str(desktop),
            "windowactivate",
            str(window_id),
        ]
    )


def _check_window_not_dropdown(window_id: int) -> bool:
    xprop_output = subprocess.check_output(["xprop", "-id", str(window_id)])

    if (
        b"_WM_WINDOW_TYPE_DROPDOWN_MENU" in xprop_output
        or b"_WM_WINDOW_TYPE_POPUP_MENU" in xprop_output
    ):
        print("Ignoring dropdown/popup type window")
        return False

    return True


def main() -> None:
    application_name_to_launch = sys.argv[1]
    application_name_to_match = (
        application_name_to_launch if len(sys.argv) - 1 < 2 else sys.argv[2]
    )

    for process in psutil.process_iter():
        process_name = process.name().split()[0]
        if process_name == application_name_to_match:
            focus_window_of_process(process.pid)
            return

    # Launch the process, since we didn't find it.
    subprocess.Popen(application_name_to_launch)


if __name__ == "__main__":
    main()
